/**************************************************************************//**
*   @file   ad5254.c
*   @brief  Implementation of AD5254 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 0
******************************************************************************/
#define AD5254_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad5254.h"
#include "opencores_i2c.h"			// I2C functions.
#include <altera_avalon_pio_regs.h>	// PIO access macros
#include <system.h>
#if (USE_AD5254 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
static unsigned char acknowledge		= 0;	// Acknowledge bit.
static unsigned char errorStatus		= 0;	// Error status.
static unsigned char serialWord			= 0;	// The 8 bits word which will be sent to the AD5254 circuit
												// as instruction and data.
static unsigned char readData			= 0;	// Read data.
static unsigned char signBit1			= 0;	// Tolerance 1 sign bit.
static unsigned char signBit2			= 0;	// Tolerance 2 sign bit.
static unsigned char eememAddress[16]	= {0x20, 0x21, 0x22, 0x23,
										   0x24, 0x25, 0x26, 0x27,
										   0x28, 0x29, 0x2A, 0x2B,
										   0x2C, 0x2D, 0x2E, 0x2F};	// Addresses of the EEMEM locations.

/*****************************************************************************/
/*  Aliases for the uC Probe data                                            */
/*****************************************************************************/
volatile unsigned char* cmdReg				= &AD5254_data[0].uchData;	// Instruction bit CMD/REG.
volatile unsigned char* zero				= &AD5254_data[1].uchData;	// Instruction bit Zero.
volatile unsigned char* eeRdac				= &AD5254_data[2].uchData;	// Instruction bit EE/RDAC.
volatile unsigned char* a4					= &AD5254_data[3].uchData;	// Instruction bit A4.
volatile unsigned char* a3					= &AD5254_data[4].uchData;	// Instruction bit A3.
volatile unsigned char* a2					= &AD5254_data[5].uchData;	// Instruction bit A2.
volatile unsigned char* a1					= &AD5254_data[6].uchData;	// Instruction bit A1.
volatile unsigned char* a0					= &AD5254_data[7].uchData;	// Instruction bit A0.
volatile unsigned char* d7					= &AD5254_data[8].uchData;	// Data bit D7.
volatile unsigned char* d6					= &AD5254_data[9].uchData;	// Data bit D6.
volatile unsigned char* d5					= &AD5254_data[10].uchData;	// Data bit D5.
volatile unsigned char* d4					= &AD5254_data[11].uchData;	// Data bit D4.
volatile unsigned char* d3					= &AD5254_data[12].uchData;	// Data bit D3.
volatile unsigned char* d2					= &AD5254_data[13].uchData;	// Data bit D2.
volatile unsigned char* d1					= &AD5254_data[14].uchData;	// Data bit D1.
volatile unsigned char* d0					= &AD5254_data[15].uchData;	// Data bit D0.
volatile unsigned char* sendDataButton		= &AD5254_data[16].uchData;	// Position of the Send Data button.
volatile unsigned char* nopR1Button			= &AD5254_data[17].uchData;	// Position of the NOP RDAC1 button.
volatile unsigned char* oneStepUpR1Button	= &AD5254_data[18].uchData;	// Position of the +1 Step RDAC1 button.
volatile unsigned char* oneStepDownR1Button	= &AD5254_data[19].uchData;	// Position of the -1 Step RDAC1 button.
volatile unsigned char* sixDbUpR1Button		= &AD5254_data[20].uchData;	// Position of the +6 dB RDAC1 button.
volatile unsigned char* sixDbDownR1Button	= &AD5254_data[21].uchData;	// Position of the -6 dB RDAC1 button.
volatile unsigned char* storeR1Button		= &AD5254_data[22].uchData;	// Position of the Store RDAC1 button.
volatile unsigned char* restoreR1Button		= &AD5254_data[23].uchData;	// Position of the Restore RDAC1 button.
volatile unsigned char* nopR3Button			= &AD5254_data[24].uchData;	// Position of the NOP RDAC3 button.
volatile unsigned char* oneStepUpR3Button	= &AD5254_data[25].uchData;	// Position of the +1 Step RDAC3 button.
volatile unsigned char* oneStepDownR3Button	= &AD5254_data[26].uchData;	// Position of the -1 Step RDAC3 button.
volatile unsigned char* sixDbUpR3Button		= &AD5254_data[27].uchData;	// Position of the +6 dB RDAC3 button.
volatile unsigned char* sixDbDownR3Button	= &AD5254_data[28].uchData;	// Position of the -6 dB RDAC3 button.
volatile unsigned char* storeR3Button		= &AD5254_data[29].uchData;	// Position of the Store RDAC3 button.
volatile unsigned char* restoreR3Button		= &AD5254_data[30].uchData;	// Position of the Restore RDAC3 button.
volatile float* 		toleranceR1Value	= &AD5254_data[31].fData;	// Tolerance RDAC1 Value.
volatile float* 		toleranceR3Value	= &AD5254_data[32].fData;	// Tolerance RDAC3 Value.
volatile unsigned char* oneStepUpButton		= &AD5254_data[33].uchData;	// Position of the +1 Step button.
volatile unsigned char* oneStepDownButton	= &AD5254_data[34].uchData;	// Position of the -1 Step button.
volatile unsigned char* sixDbUpButton		= &AD5254_data[35].uchData;	// Position of the +6 dB button.
volatile unsigned char* sixDbDownButton		= &AD5254_data[36].uchData;	// Position of the -6 dB button.
volatile unsigned char* resetButton			= &AD5254_data[37].uchData;	// Position of the Reset button.
volatile unsigned char* eememReadValue		= &AD5254_data[38].uchData;	// EEMEM Read Value
volatile unsigned char* eememWriteValue		= &AD5254_data[39].uchData;	// EEMEM Write Value
volatile unsigned char* eememSelect			= &AD5254_data[40].uchData;	// EEMEM Selector
volatile unsigned char* readEememButton		= &AD5254_data[41].uchData;	// Position of the Read EEMEM button.
volatile unsigned char* writeEememButton	= &AD5254_data[42].uchData;	// Position of the Write EEMEM button.
volatile unsigned char* wpButton			= &AD5254_data[43].uchData;	// Position of the WP button.
volatile unsigned char* loadRdac1Button		= &AD5254_data[44].uchData;	// Position of the Load RDAC1 button.
volatile unsigned char* loadRdac3Button		= &AD5254_data[45].uchData;	// Position of the Load RDAC3 button.
volatile unsigned char* loadRdac1Value		= &AD5254_data[46].uchData;	// Position of the Load RDAC1 button.
volatile unsigned char* loadRdac3Value		= &AD5254_data[47].uchData;	// Position of the Load RDAC3 button.
volatile float* 		toleranceR0Value	= &AD5254_data[48].fData;	// Tolerance RDAC0 Value.
volatile float* 		toleranceR2Value	= &AD5254_data[49].fData;	// Tolerance RDAC2 Value.
volatile unsigned char* loadRdac0Button		= &AD5254_data[50].uchData;	// Position of the Load RDAC0 button.
volatile unsigned char* loadRdac2Button		= &AD5254_data[51].uchData;	// Position of the Load RDAC2 button.
volatile unsigned char* loadRdac0Value		= &AD5254_data[52].uchData;	// Position of the Load RDAC0 button.
volatile unsigned char* loadRdac2Value		= &AD5254_data[53].uchData;	// Position of the Load RDAC2 button.
volatile unsigned char* nopR0Button			= &AD5254_data[54].uchData;	// Position of the NOP RDAC0 button.
volatile unsigned char* oneStepUpR0Button	= &AD5254_data[55].uchData;	// Position of the +1 Step RDAC0 button.
volatile unsigned char* oneStepDownR0Button	= &AD5254_data[56].uchData;	// Position of the -1 Step RDAC0 button.
volatile unsigned char* sixDbUpR0Button		= &AD5254_data[57].uchData;	// Position of the +6 dB RDAC0 button.
volatile unsigned char* sixDbDownR0Button	= &AD5254_data[58].uchData;	// Position of the -6 dB RDAC0 button.
volatile unsigned char* storeR0Button		= &AD5254_data[59].uchData;	// Position of the Store RDAC0 button.
volatile unsigned char* restoreR0Button		= &AD5254_data[60].uchData;	// Position of the Restore RDAC0 button.
volatile unsigned char* nopR2Button			= &AD5254_data[61].uchData;	// Position of the NOP RDAC2 button.
volatile unsigned char* oneStepUpR2Button	= &AD5254_data[62].uchData;	// Position of the +1 Step RDAC2 button.
volatile unsigned char* oneStepDownR2Button	= &AD5254_data[63].uchData;	// Position of the -1 Step RDAC2 button.
volatile unsigned char* sixDbUpR2Button		= &AD5254_data[64].uchData;	// Position of the +6 dB RDAC2 button.
volatile unsigned char* sixDbDownR2Button	= &AD5254_data[65].uchData;	// Position of the -6 dB RDAC2 button.
volatile unsigned char* storeR2Button		= &AD5254_data[66].uchData;	// Position of the Store RDAC2 button.
volatile unsigned char* restoreR2Button		= &AD5254_data[67].uchData;	// Position of the Restore RDAC2 button.

/**************************************************************************//**
 * @brief Initialize communication with AD8403 Board and read the tolerance.
 *
 * @param None.
 *
 * @return Success or fail.
******************************************************************************/
int AD5254_Init(void)
{
	errorStatus = 0;
	IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);							// Reset data.
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_WP);				// Output.
	*wpButton = 1;
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_WP);				// Set Write Protect pin (Active Low).
	I2C_init(OPENCORES_I2C_0_BASE, ALT_CPU_FREQ, 100000);				// Initialize I2C module.
	 // Read RDAC0 Tolerance.
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR0INT, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	readData = I2C_read(OPENCORES_I2C_0_BASE,1);			// Read the Sign and 7-bit integer value of RDAC3.
	signBit2 = (readData | 0x80);
	*toleranceR0Value = (readData & 0x7F);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR0DEC, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	*toleranceR0Value = (*toleranceR0Value * 100) +
					    I2C_read(OPENCORES_I2C_0_BASE,1);	// Read the 8-bit decimal value of RDAC3 tolerance.
	*toleranceR0Value = *toleranceR0Value / 100;
	if(signBit2)
	{
		*toleranceR0Value = *toleranceR0Value * (-1);
	}
	 // Read RDAC1 Tolerance.
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR1INT, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	readData = I2C_read(OPENCORES_I2C_0_BASE,1);			// Read the Sign and 7-bit integer value of RDAC1.
	signBit1 = (readData | 0x80);
	*toleranceR1Value = (readData & 0x7F);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR1DEC, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	*toleranceR1Value = (*toleranceR1Value * 100) +
						I2C_read(OPENCORES_I2C_0_BASE,1);	// Read the 8-bit decimal value of RDAC1 tolerance.
	*toleranceR1Value = *toleranceR1Value / 100;
	if(signBit1)
	{
		*toleranceR1Value = *toleranceR1Value * (-1);
	}
	 // Read RDAC2 Tolerance.
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR2INT, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	readData = I2C_read(OPENCORES_I2C_0_BASE,1);			// Read the Sign and 7-bit integer value of RDAC3.
	signBit2 = (readData | 0x80);
	*toleranceR2Value = (readData & 0x7F);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR2DEC, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	*toleranceR2Value = (*toleranceR2Value * 100) +
						I2C_read(OPENCORES_I2C_0_BASE,1);	// Read the 8-bit decimal value of RDAC3 tolerance.
	*toleranceR2Value = *toleranceR2Value / 100;
	if(signBit2)
	{
		*toleranceR2Value = *toleranceR2Value * (-1);
	}
	 // Read RDAC3 Tolerance.
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR3INT, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	readData = I2C_read(OPENCORES_I2C_0_BASE,1);			// Read the Sign and 7-bit integer value of RDAC3.
	signBit2 = (readData | 0x80);
	*toleranceR3Value = (readData & 0x7F);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
	errorStatus = errorStatus | acknowledge;
	I2C_write(OPENCORES_I2C_0_BASE, AD5254_TOLR3DEC, 0);
	acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
	errorStatus = errorStatus | acknowledge;
	*toleranceR3Value = (*toleranceR3Value * 100) +
						I2C_read(OPENCORES_I2C_0_BASE,1);	// Read the 8-bit decimal value of RDAC3 tolerance.
	*toleranceR3Value = *toleranceR3Value / 100;
	if(signBit2)
	{
		*toleranceR3Value = *toleranceR3Value * (-1);
	}

	return(errorStatus);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Success or fail.
******************************************************************************/
int AD5254_UpdateParameters(void)
{
	errorStatus = 0;
	if(*sendDataButton)		// If the Send Data button was pressed,
	{						// sends the value from the switches to the I2C.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		serialWord = (*cmdReg << 7) + (*zero << 6) + (*eeRdac << 5) +
					 (*a4 << 4) + (*a3 << 3) + (*a2 << 2) + (*a1 << 1) + *a0;
		I2C_write(OPENCORES_I2C_0_BASE, serialWord, 0);
		serialWord = (*d7 << 7) + (*d6 << 6) + (*d5 << 5) + (*d4 << 4) +
					    (*d3 << 3) + (*d2 << 2) + (*d1 << 1) + *d0;
		I2C_write(OPENCORES_I2C_0_BASE, serialWord, 1);
		*sendDataButton = 0;
	}
	if(*nopR0Button)		// If the NOP RDAC0 button was pressed,
	{						// sends NOP quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_NOP + 0), 1);
		*nopR0Button = 0;
	}
	if(*nopR1Button)		// If the NOP RDAC1 button was pressed,
	{						// sends NOP quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_NOP + 1), 1);
		*nopR1Button = 0;
	}
	if(*nopR2Button)		// If the NOP RDAC2 button was pressed,
	{						// sends NOP quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_NOP + 2), 1);
		*nopR2Button = 0;
	}
	if(*nopR3Button)		// If the NOP RDAC3 button was pressed,
	{						// sends NOP quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_NOP + 3), 1);
		*nopR3Button = 0;
	}
	if(*oneStepUpR0Button)	// If the +1 STEP RDAC0 button was pressed,
	{						// sends +1 STEP RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPUP + 0), 1);
		*oneStepUpR0Button = 0;
	}
	if(*oneStepUpR1Button)	// If the +1 STEP RDAC1 button was pressed,
	{						// sends +1 STEP RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPUP + 1), 1);
		*oneStepUpR1Button = 0;
	}
	if(*oneStepUpR2Button)	// If the +1 STEP RDAC2 button was pressed,
	{						// sends +1 STEP RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPUP + 2), 1);
		*oneStepUpR2Button = 0;
	}
	if(*oneStepUpR3Button)	// If the +1 STEP RDAC3 button was pressed,
	{						// sends +1 STEP RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPUP + 3), 1);
		*oneStepUpR3Button = 0;
	}
	if(*oneStepDownR0Button)// If the -1 STEP RDAC0 button was pressed,
	{						// sends -1 STEP RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPDOWN + 0), 1);
		*oneStepDownR0Button = 0;
	}
	if(*oneStepDownR1Button)// If the -1 STEP RDAC1 button was pressed,
	{						// sends -1 STEP RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPDOWN + 1), 1);
		*oneStepDownR1Button = 0;
	}
	if(*oneStepDownR2Button)// If the -1 STEP RDAC2 button was pressed,
	{						// sends -1 STEP RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPDOWN + 2), 1);
		*oneStepDownR2Button = 0;
	}
	if(*oneStepDownR3Button)// If the -1 STEP RDAC3 button was pressed,
	{						// sends -1 STEP RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_1STEPDOWN + 3), 1);
		*oneStepDownR3Button = 0;
	}
	if(*sixDbUpR0Button)	// If the +6dB RDAC0 button was pressed,
	{						// sends +6dB RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBUP + 0), 1);
		*sixDbUpR0Button = 0;
	}
	if(*sixDbUpR1Button)	// If the +6dB RDAC1 button was pressed,
	{						// sends +6dB RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBUP + 1), 1);
		*sixDbUpR1Button = 0;
	}
	if(*sixDbUpR2Button)	// If the +6dB RDAC2 button was pressed,
	{						// sends +6dB RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBUP + 2), 1);
		*sixDbUpR2Button = 0;
	}
	if(*sixDbUpR3Button)	// If the +6dB RDAC3 button was pressed,
	{						// sends +6dB RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBUP + 3), 1);
		*sixDbUpR3Button = 0;
	}
	if(*sixDbDownR0Button)	// If the -6dB RDAC0 button was pressed,
	{						// sends -6dB RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBDOWN + 0), 1);
		*sixDbDownR0Button = 0;
	}
	if(*sixDbDownR1Button)	// If the -6dB RDAC1 button was pressed,
	{						// sends -6dB RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBDOWN + 1), 1);
		*sixDbDownR1Button = 0;
	}
	if(*sixDbDownR2Button)	// If the -6dB RDAC2 button was pressed,
	{						// sends -6dB RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBDOWN + 2), 1);
		*sixDbDownR2Button = 0;
	}
	if(*sixDbDownR3Button)	// If the -6dB RDAC3 button was pressed,
	{						// sends -6dB RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_6DBDOWN + 3), 1);
		*sixDbDownR3Button = 0;
	}
	if(*storeR0Button)		// If the Store RDAC0 button was pressed,
	{						// sends Store RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_STORE + 0), 1);
		*storeR0Button = 0;
	}
	if(*storeR1Button)		// If the Store RDAC1 button was pressed,
	{						// sends Store RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_STORE + 1), 1);
		*storeR1Button = 0;
	}
	if(*storeR2Button)		// If the Store RDAC2 button was pressed,
	{						// sends Store RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_STORE + 2), 1);
		*storeR2Button = 0;
	}
	if(*storeR3Button)		// If the Store RDAC3 button was pressed,
	{						// sends Store RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_STORE + 3), 1);
		*storeR3Button = 0;
	}
	if(*restoreR0Button)	// If the Restore RDAC0 button was pressed,
	{						// sends Restore RDAC0 quick command to RDAC0.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_RESTORE + 0), 1);
		*restoreR0Button = 0;
	}
	if(*restoreR1Button)	// If the Restore RDAC1 button was pressed,
	{						// sends Restore RDAC1 quick command to RDAC1.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_RESTORE + 1), 1);
		*restoreR1Button = 0;
	}
	if(*restoreR2Button)	// If the Restore RDAC2 button was pressed,
	{						// sends Restore RDAC2 quick command to RDAC2.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_RESTORE + 2), 1);
		*restoreR2Button = 0;
	}
	if(*restoreR3Button)	// If the Restore RDAC3 button was pressed,
	{						// sends Restore RDAC3 quick command to RDAC3.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, (AD5254_RESTORE + 3), 1);
		*restoreR3Button = 0;
	}
	if(*oneStepUpButton)	// If the +1 STEP BOTH button was pressed,
	{						// sends +1 STEP BOTH quick command to both RDACs.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_1STEPUPALL, 1);
		*oneStepUpButton = 0;
	}
	if(*oneStepDownButton)	// If the -1 STEP BOTH button was pressed,
	{						// sends -1 STEP BOTH quick command to both RDACs.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_1STEPDOWNALL, 1);
		*oneStepDownButton = 0;
	}
	if(*sixDbUpButton)		// If the +6dB BOTH button was pressed,
	{						// sends +6dB BOTH quick command to both RDACs.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_6DBUPALL, 1);
		*sixDbUpButton = 0;
	}
	if(*sixDbDownButton)	// If the -6dB BOTH button was pressed,
	{						// sends -6dB BOTH quick command to both RDACs.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_6DBDOWNALL, 1);
		*sixDbDownButton = 0;
	}
	if(*resetButton)		// If the Reset BOTH button was pressed,
	{						// sends Reset BOTH quick command to both RDACs.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_RESETALL, 1);
		*resetButton = 0;
	}
	if(*readEememButton)	// If the Read EEMEM button was pressed,
	{						// reads the location selected from the EEMEM slider.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, eememAddress[*eememSelect], 0);
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 1);	// Start read sequence.
		errorStatus = errorStatus | acknowledge;
		*eememReadValue = I2C_read(OPENCORES_I2C_0_BASE,1);
		*readEememButton = 0;
	}
	if(*writeEememButton)	// If the Write EEMEM button was pressed,
	{						// update the location selected from the EEMEM slider.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, eememAddress[*eememSelect], 0);
		I2C_write(OPENCORES_I2C_0_BASE, *eememWriteValue, 1);
		*writeEememButton = 0;
	}
	if(*loadRdac0Button)	// If the Load RDAC0 button was pressed,
	{						// update RDAC0 register.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_RDAC0ADDRESS, 0);
		I2C_write(OPENCORES_I2C_0_BASE, *loadRdac0Value, 1);
		*loadRdac0Button = 0;
	}
	if(*loadRdac1Button)	// If the Load RDAC1 button was pressed,
	{						// update RDAC1 register.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_RDAC1ADDRESS, 0);
		I2C_write(OPENCORES_I2C_0_BASE, *loadRdac1Value, 1);
		*loadRdac1Button = 0;
	}
	if(*loadRdac2Button)	// If the Load RDAC2 button was pressed,
	{						// update RDAC2 register.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_RDAC2ADDRESS, 0);
		I2C_write(OPENCORES_I2C_0_BASE, *loadRdac2Value, 1);
		*loadRdac2Button = 0;
	}
	if(*loadRdac3Button)	// If the Load RDAC3 button was pressed,
	{						// update RDAC3 register.
		acknowledge = I2C_start(OPENCORES_I2C_0_BASE, AD5254_ADDRESS, 0);	// Start write sequence.
		errorStatus = errorStatus | acknowledge;
		I2C_write(OPENCORES_I2C_0_BASE, AD5254_RDAC3ADDRESS, 0);
		I2C_write(OPENCORES_I2C_0_BASE, *loadRdac3Value, 1);
		*loadRdac3Button = 0;
	}
	if(*wpButton)			// If the WP button was pressed,
	{						// toggle the state of the WP pin.
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_WP);
	}
	else
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_WP);
	}

	return (errorStatus);
}
#endif /* USE_AD5254 */
